@@ -66,7 +66,7 @@ For this to have any effect, it must be set before
 (defvar-local preview-auto--keepalive t
   "Used to keep track of when we should preview some more.")
 
-(defcustom preview-auto--extra-environments nil
+(defcustom preview-auto-extra-environments nil
   "Extra environments to consider for automatic previewing."
   :type '(repeat string))
 
@@ -105,8 +105,15 @@ Should work in AUCTeX `LaTeX-mode' buffers.  Implemented using
           (mapcar (lambda (env)
                     (cons (format "\\begin{%s}" env)
                           (cons (format "\\end{%s}" env) t)))
-                  (append texmathp-environments
-                          preview-auto--extra-environments)))
+                  (append
+                   ;; Use texmathp to get math environments.
+                   (progn (texmathp-compile)
+                          (mapcar #'car
+                                  (cl-remove-if-not
+                                   (lambda (entry)
+                                     (eq (nth 1 entry) 'env-on))
+                                   texmathp-tex-commands1)))
+                   preview-auto-extra-environments)))
          (rules (append basic-rules env-rules)))
     rules))
 
@@ -272,7 +279,11 @@ Example: (nil t t t nil nil t nil) => ((1 . 3) (6 . 6))"
     "\\\\subsubsection"
     "\\\\paragraph"
     "\\\\subparagraph")
-  "List of barrier regexps, excluded from in regions sent for previewing."
+  "List of barrier regexps, excluded from in regions sent for previewing.
+The motivation for the default value is to avoid attempting to preview
+comment regions (where functions such as texmathp occasionally fail),
+and to avoid crossing section labels, which we prefer to display using
+tex-fold rather than preview."
   :type '(repeat string))
 
 (defun preview-auto--get-valid-region (beg end search-from-beginning)
